#!/usr/bin/env python3


"""Represent one leg of a public transport trip."""


from .trip_leg import TripLeg


__all__ = ["TransitLeg"]


class TransitLeg(TripLeg):
    """Represent one leg of a public transport trip."""
